/*
 * InAppBilling Module Example App
 *
 * Developed for Appcelerator by:
 *   Alexander Conway
 *   Logical Labs, LLC
 *
 * Please direct all questions, feedback, and concerns to info@appcelerator.com.
 *
 * This example app demonstrates the functionality of the InAppBilling Module
 * and the Android In-App Billing Service.
 *
 * NOTE 1:
 *
 * In-App Billing cannot run in the Android emulator!
 *
 * NOTE 2:
 *
 * The example currently provides only the 4 android.test purchase requests.
 * With these static requests the restoreTransactions function will not work,
 * also adding a developerPayload will do nothing. Additional 'real' products can be
 * added to the example by following the steps listed here:
 * http://developer.android.com/guide/market/billing/billing_testing.html
 * (See 'Testing In-app Purchases Using Your Own Product IDs')
 * and adding the product IDs to the picker.
 *
 */

////////////////////////////////////
//Module Implementation
////////////////////////////////////

/*
 * First include the Module
 */

var InAppBilling = require('ti.inappbilling');

/*
 * Set Public Key for signature verification
 * (Not required: If not set, module won't verify)
 */

//InAppBilling.setPublicKey('<< YOUR KEY HERE >>');


////////////////////////////////////
//UI Setup
////////////////////////////////////


var win = Ti.UI.createWindow({
	backgroundColor: 'white',
	title:'InAppBilling Example',
	fullscreen:false,
	exitOnClose:true
});

var scrollView = Ti.UI.createScrollView({
	layout:'vertical',
	left:0,
	right:0,
	top:0,
	bottom:0
});

var checkBillingSupportedButton = Ti.UI.createButton({
	title: 'Check Billing Supported',
	left:20,
	right:20,
	height:60,
	top:10
});

var restoreTransactionsButton = Ti.UI.createButton({
	title: 'Restore Transactions',
	enabled: false,
	left:20,
	right:20,
	height:60,
	top:10
});


var buyButton = Ti.UI.createButton({
	title: 'Buy',
	enabled: false,
	left:20,
	right:20,
	height:60,
	top:10
});

var picker = Ti.UI.createPicker({
	width: 180,
	selectionIndicator: true,
	left:20,
	right:20,
	height:60,
	top:10
});

picker.add([
	Ti.UI.createPickerRow({ title: 'android.test.purchased'}),
	Ti.UI.createPickerRow({ title: 'android.test.canceled' }),
	Ti.UI.createPickerRow({ title: 'android.test.refunded' }),
	Ti.UI.createPickerRow({ title: 'android.test.item_unavailable' })
]);

var addPayloadButton = Ti.UI.createButton({
	title: 'Add Payload',
	left:20,
	right:20,
	height:60,
	top:10
});

var devPayload = '';
	
scrollView.add(checkBillingSupportedButton);
scrollView.add(restoreTransactionsButton);
scrollView.add(buyButton);
scrollView.add(picker);
scrollView.add(addPayloadButton);

win.add(scrollView);
win.open();

picker.setSelectedRow(0, 0, true);

function NotifyMe(text){
	Ti.API.info('< -- > '+text);
	Ti.UI.createNotification({
		message:text,
		duration: Ti.UI.NOTIFICATION_DURATION_SHORT
	}).show();
}

NotifyMe('Open DDMS to see the logs');

/*
 * Helper functions: translate InAppBilling response codes to Strings
 *
 *	InAppBilling response constants
 *	
 *	RESULT_OK
 *	RESULT_USER_CANCELED
 *	RESULT_SERVICE_UNAVAILABLE
 *	RESULT_BILLING_UNAVAILABLE
 *	RESULT_ITEM_UNAVAILABLE
 *	RESULT_DEVELOPER_ERROR
 *	RESULT_ERROR
 *	
 */
function ResponseString(responseCode) {
	switch (responseCode) {
		case InAppBilling.RESULT_OK:
			return 'OK';
		case InAppBilling.RESULT_USER_CANCELED:
			return 'USER CANCELED';
		case InAppBilling.RESULT_SERVICE_UNAVAILABLE:
			return 'SERVICE UNAVAILABLE';
		case InAppBilling.RESULT_BILLING_UNAVAILABLE:
			return 'BILLING UNAVAILABLE';
		case InAppBilling.RESULT_ITEM_UNAVAILABLE:
			return 'ITEM UNAVAILABLE';
		case InAppBilling.RESULT_DEVELOPER_ERROR:
			return 'DEVELOPER ERROR';
		case InAppBilling.RESULT_ERROR:
			return 'RESULT ERROR';
	}
	return '';
}
/*
 *	InAppBilling verification constants
 *	
 *	SIGNATURE_VERIFIED
 *	NULL_DATA
 *	SIGNATURE_ERROR
 *	UNKNOWN_NONCE
 *	PUBLIC_KEY_NULL
 *	
 */

function VerificationString(verificationCode) {
	switch (verificationCode) {
		case InAppBilling.SIGNATURE_VERIFIED:
			return 'SIGNATURE VERIFIED';
		case InAppBilling.NULL_DATA:
			return 'NULL DATA';
		case InAppBilling.SIGNATURE_ERROR:
			return 'SIGNATURE ERROR';
		case InAppBilling.UNKNOWN_NONCE:
			return 'UNKNOWN NONCE';
		case InAppBilling.PUBLIC_KEY_NULL:
			return 'PUBLIC KEY NULL';
	}
	return '';
}


addPayloadButton.addEventListener('click', function () {
	var root = Ti.UI.createView({});
	var view = Ti.UI.createView({
		width: 300,
		height: '100'
	});
	var textField = Ti.UI.createTextField({
		width: 300,
		height: 50,
		value: devPayload
	});
	root.add(view);
	view.add(textField);
	var dialog = Ti.UI.createOptionDialog({
		title: 'Add Developer Payload to Item',
		options: null,
		buttonNames: ['OK', 'CANCEL'],
		androidView: root
		
	});
	dialog.show();
	dialog.addEventListener('click', function(e){
		if (e.index == 0) {
			devPayload = textField.value;
		}
	});
});

/*
 * 
 * Have the 'Check Billing Supported' button initialize the billing service
 */

checkBillingSupportedButton.addEventListener('click', function(e){
	InAppBilling.startBillingService();

	NotifyMe('Starting Billing Service');
});


buyButton.addEventListener('click', function(e){

	InAppBilling.requestPurchase({
		productId: picker.getSelectedRow(0).title,
		developerPayload: devPayload
	});

	NotifyMe('Request Purchase ');
});

restoreTransactionsButton.addEventListener('click', function(e){


	NotifyMe('Restore Transactions');

	InAppBilling.restoreTransactions();
});


/*
 * Event listeners constants
 *
 *	ON_BIND_EVENT
 *	ON_CONNECT_EVENT
 *	RESPONSE_EVENT
 *	PURCHASE_STATE_CHANGED_EVENT
 *	NOTIFY_EVENT
 *
 *
 */


InAppBilling.addEventListener(InAppBilling.ON_BIND_EVENT, function(e){
	if (e.result == InAppBilling.SUCCESS) {
		NotifyMe('Billing Service Bound');
	} else {
		NotifyMe('Billing Service Bind Failed');
	}
});

InAppBilling.addEventListener(InAppBilling.ON_CONNECT_EVENT, function(e){
	InAppBilling.checkBillingSupported();
	buyButton.enabled = true;
	restoreTransactionsButton.enabled = true;
});

InAppBilling.addEventListener(InAppBilling.RESPONSE_EVENT, function(e){
	NotifyMe('RESPONSE CALLED ' + e.requestId + e.responseCode);
	if (e.sync == true) {
		var response = ResponseString(e.responseCode);

		NotifyMe('Request Id: ' + e.requestId+ '\n'+'Response code: ' + e.responseCode);

		Ti.API.info('Request Id: ' + e.requestId);
		Ti.API.info('Response code: ' + e.responseCode);
	} else {
		Ti.API.info('RESPONSE CALLED \n' + 'Request Id:\n' + e.requestId + ' ' + '\nResponse Code:' + ResponseString(e.responseCode));
	}
});

InAppBilling.addEventListener(InAppBilling.PURCHASE_STATE_CHANGED_EVENT, function(e){
	
	NotifyMe('PURCHASE STATE CHANGED CALLED ' + e.signedData + ' ' + e.signature+'\n'+ 'SECURITY RESULT ' + e.result);
	
	Ti.API.info('PURCHASE STATE CHANGED CALLED');
	Ti.API.info('Signature Verification Result:\n' + VerificationString(e.result));
	Ti.API.info('Signed Data:\n' + e.signedData);
	
	if (e.signedData != null) {
		var response = JSON.parse(e.signedData);
		InAppBilling.confirmNotifications({
			notificationIds: [response.orders[0].notificationId]
		});
	}
});

InAppBilling.addEventListener(InAppBilling.NOTIFY_EVENT, function(e){
	
	Ti.API.info('NOTIFY CALLED \n' + 'Notify Id:\n' + e.notifyId);
	
	InAppBilling.getPurchaseInformation({
		notificationIds: [e.notifyId]
	});
});